/*
    Copyright 2014 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

VC\bin\vcvars32.bat
VC\bin\cl.exe steamlobbylist.cpp /I ..\sdk\public ..\sdk\redistributable_bin\steam_api.lib /Ox
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <windows.h>
#pragma comment(lib, "ws2_32.lib")

#define STEAM_API_NODLL
#include "steam/steam_api.h"



#define VER     "0.1a"



void show_steamid_info(char *info, CSteamID id) {
    printf("%s %llu - %u %u %u %u\n", info,
        id.ConvertToUint64(),
        id.GetAccountID(), id.GetUnAccountInstance(), id.GetEAccountType(), id.GetEUniverse());
}        



void show_appid_name(AppId_t appID) {
    char    name[1024],
            *p;

    if(SteamAppList()->GetAppName(appID, name, sizeof(name)) < 0)
    if(SteamAppList()->GetAppInstallDir(appID, name, sizeof(name)) < 0)
    if(SteamApps()->GetAppInstallDir(appID, name, sizeof(name)) <= 0)
        name[0] = 0;
    p = strrchr(name, '\\');
    if(!p) p = strrchr(name, '/');
    if(!p) p = name;
    else p++;
    printf("\n### GAME %u - %s ###\n", (int)appID, p);
}



class CLobbyBrowser
{
public:
    int Refresh(bool check_if_terminated) {
        if(check_if_terminated) {
            if((time(NULL) - timestamp) > 10) return 1;   // timeout
            return done;
        }
        
        timestamp = time(NULL);
        done = 0;
		SteamAPICall_t hSteamAPICall = SteamMatchmaking()->RequestLobbyList();
		m_SteamCallResultLobbyMatchList.Set( hSteamAPICall, this, &CLobbyBrowser::OnLobbyMatchListCallback );
        return 0;
    }
private:
    bool    done;
    time_t  timestamp;
	CCallResult<CLobbyBrowser, LobbyMatchList_t> m_SteamCallResultLobbyMatchList;
    void OnLobbyMatchListCallback( LobbyMatchList_t *pCallback, bool bIOFailure ) {
        CSteamID steamIDLobby;
        int     datas;
        char    Key[1024],      // bigger than usual
                Value[8192];    // bigger than usual

        printf("%d lobb%s\n", pCallback->m_nLobbiesMatching, (pCallback->m_nLobbiesMatching == 1) ? "y" : "ies");
        for ( int iLobby = 0; iLobby < pCallback->m_nLobbiesMatching; iLobby++ )
        {
            steamIDLobby = SteamMatchmaking()->GetLobbyByIndex( iLobby );
            show_steamid_info("\nlobby", steamIDLobby);
            int cLobbyMembers = SteamMatchmaking()->GetNumLobbyMembers( steamIDLobby );
            datas = SteamMatchmaking()->GetLobbyDataCount(steamIDLobby);
            for(int iLobbyData = 0; iLobbyData < datas; iLobbyData++) {
                memset(Key, 0, sizeof(Key));        // not needed
                memset(Value, 0, sizeof(Value));    // not needed
                SteamMatchmaking()->GetLobbyDataByIndex(steamIDLobby, iLobbyData, Key, sizeof(Key), Value, sizeof(Value));
                printf("  %s: %s\n", Key, Value);
            }
        }
        done = 1;
    }
};



extern "C" void __cdecl SteamAPIDebugTextHook( int nSeverity, const char *pchDebugText ) {
	fprintf(stderr, "%s\n", pchDebugText);
}



int main(int argc, char *argv[]) {
    static AppId_t installed_apps[1000000];
    CLobbyBrowser *m_pLobbyBrowser = NULL;
    CSteamID m_SteamIDLocalUser;
    AppId_t appID       = 0,
            appID_next;
    int     init        = 0,
            do_range    = 0,
            installed_apps_scan = 0;
    char    *arg;

    fprintf(stderr,
        "\n"
        "steamlobbylist %s\n"
        "by Luigi Auriemma\n"
        "e-mail: me@aluigi.org\n"
        "web:    aluigi.org\n"
        "\n", VER);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <appID*>\n"
            "\n"
            "Use appID equal to \"\" or ALL to list all the lobbies of the installed games\n"
            "Use appID with a FROM-TO to scan all the id in that range\n"
            "You can also use the comma as separator\n"
            "Example: 1-100,120,480\n"
            "\n"
            "Some games allow you to query their lobbies even if you don't own them.\n"
            "\n", argv[0]);
        exit(1);
    }
    
    arg = argv[1];    
    if(!arg[0] || (arg[0] == '-') || !stricmp(arg, "ALL")) {
        installed_apps_scan = -1;
        appID = 480;
        arg = NULL;
    }

redo:
    if(installed_apps_scan > 0) appID = installed_apps[--installed_apps_scan];
    if(!installed_apps_scan) {
        if(do_range) {
            if(appID_next < appID) appID--;
            else if(appID_next > appID) appID++;
            else do_range = 0;
        }
        if(!do_range) {
            if(!arg || !arg[0]) return 0;   // end of list
            #define SKIP_NUMBER     while(*arg && (*arg > ' ') && (*arg != ',') && (*arg != '-')) arg++;
            #define SKIP_SEPARATORS while(*arg && !((*arg > ' ') && (*arg != ',') && (*arg != '-'))) arg++;
            appID = atoi(arg);
            SKIP_NUMBER
            if(*arg == '-') {
                do_range = 1;
                SKIP_SEPARATORS
                appID_next = atoi(arg);
                SKIP_NUMBER
            }
            SKIP_SEPARATORS
        }
    }

    char tmp[32];
    sprintf(tmp, "%u", appID);
    SetEnvironmentVariable("SteamAppId", tmp);
    SetEnvironmentVariable("SteamGameId", tmp);
    
    if(!SteamAPI_Init()) {
        printf("\nError: Steam is not running\n");
        exit(1);
    }
    SteamClient()->SetWarningMessageHook( &SteamAPIDebugTextHook );
    if(!init) {
        init = 1;
        if(!SteamUser()->BLoggedOn()) printf("Steam user is not logged in\n");
        m_SteamIDLocalUser = SteamUser()->GetSteamID();
        show_steamid_info("Your ID", m_SteamIDLocalUser);
        printf("GetPersonaName %s\n", SteamFriends()->GetPersonaName());
    }

    if(installed_apps_scan < 0) {
        installed_apps_scan = SteamAppList()->GetInstalledApps(installed_apps, sizeof(installed_apps) / sizeof(AppId_t));
        goto quit;
    }

    show_appid_name(appID);

    printf("- request list of lobbies:\n");    
    if(!m_pLobbyBrowser) m_pLobbyBrowser = new CLobbyBrowser();
    m_pLobbyBrowser->Refresh(0);
    while(!m_pLobbyBrowser->Refresh(1)) {
        SteamAPI_RunCallbacks();
        Sleep(10);
    }

quit:
    //SteamController()->Shutdown();
    //SteamClient()->BShutdownIfAllPipesClosed();
    SteamAPI_Shutdown();
    goto redo;
    return 0;
}
